import React, { Component } from 'react'
import { TextField } from '@material-ui/core'
import { Field, Form, Store } from '../src'
import styles from './styles.icss'

//	Our main component
class Formtest extends Component {

	constructor(props) {
		super(props)
		const me = this

		//	Basic example of un/redo
		const maxHistory = 100
		me.values = []
		me.valueIndex = -1
		me.isUnRedo = false

		//	Set initial values of the store here
		this.store = new Store({
			name: 'Test name'
		},
		(values) => {
			if (!me.isUnRedo) {
				//	Set past value here...
				me.values.push(JSON.parse(JSON.stringify(values)))
				//	Remove old entries, if too big
				if (me.values.length > maxHistory) {
					me.values.shift()
					me.valueIndex -= 1
				}

				me.valueIndex += 1
			} else {
				me.isUnRedo = false
			}
			/* eslint-disable no-underscore-dangle */
			if (me._isMounted) {
				me.forceUpdate()
			}
		})

		this.undo = this.undo.bind(this)
		this.redo = this.redo.bind(this)
		this.canUndo = this.canUndo.bind(this)
		this.canRedo = this.canRedo.bind(this)
	}

	componentDidMount() {
		/* eslint-disable no-underscore-dangle */
		this._isMounted = true
	}

	componentWillUnmount() {
		this._isMounted = false
	}

	undo() {
		this.valueIndex -= 1
		if (this.valueIndex < 0) {
			this.valueIndex = 0
		}
		this.store.clear(true)
		this.isUnRedo = true
		this.store.set(this.values[this.valueIndex])
	}

	redo() {
		this.valueIndex += 1
		if (this.valueIndex > (this.values.length - 1)) {
			this.valueIndex = this.values.length - 1
		}
		this.store.clear(true)
		this.isUnRedo = true
		this.store.set(this.values[this.valueIndex])
	}

	canUndo() {
		return this.values && this.values.length > 0 && this.valueIndex > 0
	}

	canRedo() {
		return this.values && this.values.length > 0 && this.valueIndex < (this.values.length - 1)
	}

	render() {
		return (
			<div className="route route__signup">
				<h1>Form with undo</h1>

				<div className={styles.data}>
					<h2>Data model</h2>
					<pre>{JSON.stringify(this.store.get(), null, 4)}</pre>
				</div>

				<button type="button" disabled={!this.canUndo()} onClick={this.undo}>Undo</button>
				<button type="button" disabled={!this.canRedo()} onClick={this.redo}>Redo</button>

				<Form store={this.store}>

					<div>
						<Field
							name="name"
							placeholder="Name"
							component={TextField}
						/>

						<Field
							name="otherfield"
							placeholder="Other"
							component={TextField}
						/>
					</div>

				</Form>

			</div>
		)
	}
}

export default Formtest
