import equivalent from './equivalent'

const debounce = (func, duration = 100) => {
	let promise
	let lastArgs
	let later
	let timeout
	return (...args) => {
		if (promise) {
			if (equivalent(args, lastArgs)) {
				return promise
			}
			promise.reject()
			promise = null
		}
		clearTimeout(timeout)
		lastArgs = args
		if (!promise) {
			let rejectCallback
			promise = new Promise((resolve, reject) => {
				rejectCallback = reject
				later = () => {
					resolve(func.apply(this, lastArgs))
					promise = null
				}
			})
			promise.reject = rejectCallback
		}
		timeout = setTimeout(later, duration)
		return promise
	}
}

export default debounce
